#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov 11 10:29:35 2024

@author: anton
"""

import SLiCAP as sl 
import sympy as sp

sl.initProject("SLiCAP_LTD")

cir = sl.makeCircuit("kicad/SLiCAP_LTD/SLiCAP_LTD.kicad_sch")

# Symbolic pole-zero analysis

PZ_sym = sl.doPZ(cir)

# Print the DC gain:
    
print("\nDC gain:")
sp.pprint(PZ_sym.DCvalue)

# Print the zeros

print("\nZeros:")
for zero in PZ_sym.zeros:
    sp.pprint(zero)

# Print the poles of the compensated probe (C_i = C_s*R_se/R_i)    

# Simplification of the expression for the poles is not easy to do. Hence, it
# easier to redefine the value of C1 or C2. A function for doing this has not 
# been implemented because currently, element values are only checked when
# generating  the netlist. However, below is the method that works, but be 
# careful! In this way you can change the type of the element (R, L, C), or 
# make errors in expressions that are not checked.

# The value of an element is stored in its parameter "value". 
# We will store the old value first:

C2_value = cir.elements["C2"].params["value"]

# Now redefine the value
cir.elements["C2"].params["value"] = sp.sympify("C_c*R_se/R_i")

# And run the analysis again with the new value
PZ_sym_comp = sl.doPZ(cir)

print("\nPoles (compensated):")
for pole in PZ_sym_comp.poles:
    sp.pprint(pole, wrap_line=False)
    
# Restore the old value of C2:
cir.elements["C2"].params["value"] = C2_value

# Numeric pole-zero analysis with canceling of conciding poles and zeros
PZ_num_comp = sl.doPZ(cir, pardefs = "circuit")

# listPZ() displays the poles in Hz if sl ini.hz=True
sl.listPZ(PZ_num_comp)

# Numeric pole analysis (no pole-zero compensation)
Poles_num_comp = sl.doPoles(cir, pardefs = "circuit")
sl.listPZ(Poles_num_comp)

# Numeric zero analysis (no pole-zero compensation)
Zeros_num_comp = sl.doZeros(cir, pardefs = "circuit")
sl.listPZ(Zeros_num_comp)

# Root - locus plot stepping C_i (also plotting the zero)
step_dict = {'params': 'C_i',
             'method': 'lin',
             'start':  '50p',
             'stop':   '150p',
             'num':    11}
gain_poles_step_C_i = sl.doPoles(cir, pardefs="circuit", numeric=True, stepdict=step_dict)
gain_zero           = sl.doZeros(cir, pardefs="circuit", numeric=True)
rl_C_i = sl.plotPZ("rl_C_i", "Root locus step $C_i$", 
                   [gain_poles_step_C_i, gain_zero], xmin=-3e3, xmax=-1e3, 
                   ymin=-1, ymax=1, show=True)

# Try yourself: Root-locus plot stepping R_s

# Symbolic Transfer
sym_transfer = sl.doLaplace(cir)
print("\nSymbolic transfer:")
sp.pprint(sym_transfer.laplace, wrap_line=False)

# Symbolic coefficients of differential equation
gain, numer_coeffs, denom_coeffs = sl.coeffsTransfer(sym_transfer.laplace)
print("\nGain:")
sp.pprint(gain)
print("\nCoefficients numerator (symbolic, ascendeng order):")
for coeff in numer_coeffs:
    sp.pprint(coeff)
print("\nCoefficients denomiator (symbolic, ascendeng order):")
for coeff in denom_coeffs:
    sp.pprint(coeff)

# Numeric coefficients of differential equation
num_transfer = sl.doLaplace(cir, pardefs="circuit", numeric=True)
gain, numer_coeffs, denom_coeffs = sl.coeffsTransfer(num_transfer.laplace)
print("\nGain:")
sp.pprint(gain)
print("\nCoefficients numerator (numeric, ascendeng order):")
for coeff in numer_coeffs:
    sp.pprint(coeff)
print("\nCoefficients denomiator (numeric, ascendeng order):")
for coeff in denom_coeffs:
    sp.pprint(coeff)

# Bode-plots with stepping of C_i
gain_laplace_step_C_i = sl.doLaplace(cir, pardefs="circuit", numeric=True, stepdict=step_dict)
dBmag = sl.plotSweep("dBmag", "dB Magnitude step $C_i$", 
                     gain_laplace_step_C_i, 1, 1e9, 100, show=True)
phase = sl.plotSweep("phase", "Phase step $C_i$", 
                     gain_laplace_step_C_i, 1, 1e9, 100, funcType="phase", show=True)

# Symbolic step response of compensated probe
cir.elements["C2"].params["value"] = sp.sympify("C_c*R_se/R_i")
sym_step = sl.doStep(cir)
print("\nUnit step response:")
sp.pprint(sym_step.stepResp, wrap_line=False)

# Restore the old value of C2:
cir.elements["C2"].params["value"] = C2_value

# Unit step responses
gain_step_step_C_i = sl.doStep(cir, pardefs="circuit", numeric=True, stepdict=step_dict)
steResp = sl.plotSweep("stepResp", "Unit step response, step $C_i$.", 
                       gain_step_step_C_i, 0, 500, 500, sweepScale="u", show=True)
steRespZoom = sl.plotSweep("stepResp", "Unit step response, step $C_i$.", 
                       gain_step_step_C_i, 0, 2.5, 100, sweepScale="n", show=True)